from unittest import TestCase

from django.template import Template, Context
from django.test.client import Client
from BeautifulSoup import BeautifulSoup, Tag

from pml.utils import remove_comments, swap_tags, html_to_text


class PummelUtilTestCase(TestCase):

    def test_remove_comments(self):
        soup = BeautifulSoup('<!-- hello world --><b>bold</b>')
        self.assertEqual('<b>bold</b>',
                         remove_comments(soup).renderContents())

    def test_swap_tags(self):
        soup = BeautifulSoup('<b>bold</b>')
        soup = swap_tags(soup, [('b', 'strong')])
        self.assertEqual('<strong>bold</strong>',
                         soup.renderContents())

    def test_swap_tags_with_callable(self):
        def replace_with_p(soup, tag):
            replacement_tag = Tag(soup, 'p')
            replacement_tag.insert(0, '- %s' % tag.text)
            return replacement_tag

        soup = BeautifulSoup('<li>list</li>')
        soup = swap_tags(soup, [('li', replace_with_p)])
        self.assertEqual('<p>- list</p>',
                         soup.renderContents())

    def test_html_to_text(self):
        text = html_to_text(
            """
            <a href="#">hello world</a>
            <ul>
                <li>list item 1</li>
                <li>list item 2</li>
                <li>list item 3</li>
            </ul>
            """)
        self.assertEqual(text.strip(), (
            "<LINK href=\"#\">hello world</LINK>\n"
            "<p>"
            "- list item 1<br />"
            "- list item 2<br />"
            "- list item 3<br />"
            "</p>").strip())


class PummelTemplateTagTestCase(TestCase):

    def render_template(self, template_string, context):
        return Template(template_string).render(Context(context))

    def test_text_module_tag(self):
        response = self.render_template("""
            {% load pml_inclusion_tags %}
            {% spaceless %}
                {% text_module html="{{foo}}" %}
            {% endspaceless %}
            """, {'foo': 'bar'})
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT><p>{{foo}}</p></TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))

    def test_text_module_tag_with_var_as_html(self):
        response = self.render_template("""
            {% load pml_inclusion_tags %}
            {% spaceless %}
                {% with foo='bar' %}
                {% text_module html=foo %}
                {% endwith %}
            {% endspaceless %}
            """, {})
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT><p>bar</p></TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))

    def test_text_module_tag_with_template(self):
        response = self.render_template("""
            {% load pml_inclusion_tags %}
            {% spaceless %}
                {% text_module template="{{foo}}" %}
            {% endspaceless %}
            """, {'foo': 'bar'})
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT><p>bar</p></TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))

    def test_text_module_tag_with_link(self):
        response = self.render_template("""
            {% load pml_inclusion_tags %}
            {% spaceless %}
                {% text_module template="<small><a href='/about/'>About</a></small>" %}
            {% endspaceless %}
            """, {})
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT><p><small><LINK href="/about/">About</LINK></small></p></TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))

    def test_text_module_tag_with_html(self):
        response = self.render_template("""
            {% load pml_inclusion_tags %}
            {% spaceless %}
                {% text_module template="<small>&copy; {{year}}</small>" %}
            {% endspaceless %}
            """, {'year': 2013})
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT><p><small>&#169; 2013</small></p></TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))

    def test_text_module_tag_for_download(self):
        response = self.render_template("""
            {% load pml_inclusion_tags %}
            {% spaceless %}
                {% text_include template='pml/tests/download_inline_block.html' %}
            {% endspaceless %}
            """, {})
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT><p>'
            '<LINK href="http://yal.pf.org/download/">'
            'Click here to download This Amazing download now!!!!'
            '</LINK>'
            '</p></TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))

    def test_page_banner_include(self):
        response = self.render_template("""
            {% load pml_inclusion_tags %}
            {% spaceless %}
                {% page_banner_include image_url='/some/image/' template='pml/tests/object_detail_header_include.html' %}
            {% endspaceless %}
            """, {})

        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<GALLERY>'
            '<ITEM>'
            '<IMAGE href="/some/image/" float="true" align="" mime-type="image/jpeg" />'
            '<TEXT><p><color value="#E40BBA">Title</color><br />'
            '<color value="#666666">Published</color>'
            '</p></TEXT>'
            '</ITEM>'
            '</GALLERY>'
            '</CONTAINER>'
            '</MODULE>'
        ))

    def test_thumbnail_include_tag(self):
        response = self.render_template("""
            {% load pml_inclusion_tags %}
            {% spaceless %}
                {% thumbnail_include image_url='#' template="pml/tests/modelbase_block_include.html" %}
            {% endspaceless %}
            """, {})
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<GALLERY>'
            '<ITEM>'
            '<IMAGE href="#" float="true" align="left" mime-type="image/jpeg" />'
            '<TEXT><p>'
            '<LINK href="/link/">Title</LINK><br />'
            '<small>Published date</small><br />'
            '<small>Comments</small>'
            '</p></TEXT>'
            '</ITEM>'
            '</GALLERY>'
            '</CONTAINER>'
            '</MODULE>'
        ))


    def test_clickable_banner_tag(self):
        response = self.render_template("""
            {% load pml_inclusion_tags %}
            {% spaceless %}
                {% clickable_banner_include image_url='#' link_url='/live/' template="pml/tests/promo_block_include.html" %}
            {% endspaceless %}
            """, {})
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT>'
            '<LINK href="/live/">'
            '<IMAGE href="#" align="full" type="image/jpeg"/>'
            '<TEXT><p><LINK href="/live/">This</LINK> is a promo description\n'
            '</p></TEXT>'
            '</LINK>'
            '</TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))

    def test_html_sanitize(self):
        def render_html(html):
            return self.render_template("""
                {% load pml_inclusion_tags %}
                {% spaceless %}
                    {% text_module template="{{html|safe}}" %}
                {% endspaceless %}
                """, {'html': html})
        response = render_html('&ldquo;Hello&rdquo;')
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT>'
            '<p>"Hello"</p>'
            '</TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))
        response = render_html('&lsquo;Hello&rsquo;')
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT>'
            '<p>\'Hello\'</p>'
            '</TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))
        response = render_html('&quot;Hello&quot;')
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT>'
            '<p>"Hello"</p>'
            '</TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))
        response = render_html('Hello&ndash;World')
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT>'
            '<p>Hello-World</p>'
            '</TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))
        response = render_html('Hello&ndash;World')
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT>'
            '<p>Hello-World</p>'
            '</TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))
        response = render_html('Hello&hellip;')
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT>'
            '<p>Hello...</p>'
            '</TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))
        response = render_html('Hello&raquo;')
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT>'
            '<p>Hello&#187;</p>'
            '</TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))
        response = render_html('Hello&laquo;')
        self.assertEqual(response.strip(), (
            '<MODULE>'
            '<CONTAINER type="data">'
            '<TEXT>'
            '<p>Hello&#171;</p>'
            '</TEXT>'
            '</CONTAINER>'
            '</MODULE>'
        ))



class PummeliMiddlewareTestCase(TestCase):
    def test_redirect_respects_ignore_paths(self):
        client = Client()
        response = client.get('/')
        self.assertEquals(response['Content-Type'], 'text/xml')

        response = client.get('/admin/')
        self.assertEquals(response['Content-Type'], 'text/html; charset=utf-8')

        response = client.get('/redirect/')
        self.assertEquals(response['Content-Type'], 'text/html; charset=utf-8')



import random

from django.conf import settings
from django.contrib.auth.middleware import RemoteUserMiddleware
from django.contrib.messages.api import get_messages
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect, HttpResponsePermanentRedirect
from django.utils.cache import patch_cache_control
from lxml import etree


class SkipMixin(object):
    def should_skip(self, request):
        path_bypass = [
            reverse('admin:index'),  # Skip for admin
            settings.STATIC_URL,  # Skip for static
            settings.MEDIA_URL,  # Skip for media
        ]

        if hasattr(settings, 'PML_IGNORE_PATH'):
            path_bypass += settings.PML_IGNORE_PATH

        for path in path_bypass:
            if request.path.startswith(path):
                return True
        return False


class FormMiddleware(object):
    def process_request(self, request):
        if request.GET.get('submit'):
            request.method = "POST"
            request.POST = request.POST.copy()
            request.POST.update(request.GET)


class NoCacheMiddleware(SkipMixin, object):
    def process_response(self, request, response):
        if self.should_skip(request):
            return response

        # Set HTTP response cache headers to not cache.
        patch_cache_control(response, no_cache=True, no_store=True, must_revalidate=True)
        return response


class RandomLinkMiddleware(SkipMixin, object):
    def process_response(self, request, response):
        if self.should_skip(request):
            return response

        # Bypass further processing for empty(redirect) response.
        if not response.content:
            return response

        # Add a random get param to all LINK hrefs to further prevent
        # Vodafone gateway from caching.
        try:
            tree = etree.fromstring(response.content)
        except etree.XMLSyntaxError:
            return response
        for link in tree.findall('.//LINK'):
            href = link.get('href')
            if href:
                if '?' in href:
                    href += '&rnd=%s' % random.randint(1000000, 9999999)
                else:
                    href += '?rnd=%s' % random.randint(1000000, 9999999)
                link.set('href', href)
        response.content = etree.tostring(tree)
        return response


class RedirectMiddleware(SkipMixin, object):
    def process_response(self, request, response):
        if self.should_skip(request):
            return response

        if response.__class__ in [HttpResponseRedirect, HttpResponsePermanentRedirect, ]:
            location = response._headers['location'][1]
            if not location.startswith('/admin'):
                url = response._headers['location'][1]
                if '?' in url:
                    url += '&rnd=%s' % str(random.randint(1000000, 9999999))
                else:
                    url += '?rnd=%s' % random.randint(1000000, 9999999)
                return render_to_response('pml/redirect.xml', {'url': url, 'messages': get_messages(request)}, mimetype='text/xml')
        return response


class VLiveRemoteUserMiddleware(RemoteUserMiddleware):
    header = 'HTTP_X_UP_CALLING_LINE_ID'


class XMLResponseMiddleware(SkipMixin, object):
    """
    Sets response mimetype to text/xml.
    """
    def process_response(self, request, response):
        if self.should_skip(request):
            return response

        # When developing on error show trace page
        if settings.DEBUG and response.status_code in [500, ]:
            return response

        # Set xml content type for everything else.
        response['Content-Type'] = 'text/xml'
        return response



import tidylib
tidylib.BASE_OPTIONS['preserve-entities'] = True

from BeautifulSoup import BeautifulSoup, Comment, Tag

from pml.constants import COLORS


class InvalidColorException(Exception):
    pass


def resolve_color(color):
    """
    Resolves a provided simple color, i.e. 'red' to its PML equivalent,
    i.e. MiRed
    @param color string: simple color
    @return string: resolved PML color or InvalidColorException if color
        could not be resolved, or None if provided color is None.
    """
    if color:
        if color.lower() not in COLORS:
            raise InvalidColorException()
    else:
        return None

    return 'Mi%s' % color.title()


def gen_context(color):
    """
    Generates a context containing base module attributes
    @param color string: module background color
    @return dict: context dictionary
    """
    return {
        'color': resolve_color(color),
    }


def html_to_text(html):
    """
    Evaluates provided HTML and converts result to PML for TEXT containers.
    @param html string: html to convert
    @param Context context: context to be used for html evaluation
    @return string: evaluated HTML converted to TEXT appropriate PML
    """

    def fake_ul(soup, tag):
        # Fake UL elements by inserting a p with a prefixed dash for
        # each LI
        p_tag = Tag(soup, 'p')
        insertables = []
        for li in tag.findAll('li'):
            insertables.append('- %s' % (li.text,))
            insertables.append(Tag(soup, 'br'))

        for index, element in enumerate(insertables):
            p_tag.insert(index, element)
        return p_tag

    def convert_anchor(soup, tag):
        link_tag = Tag(soup, 'LINK')
        try:
            link_tag['href'] = tag['href']
        except KeyError:
            pass

        link_tag.insert(0, tag.text)
        return link_tag

    def convert_font(soup, tag):
        try:
            color_tag = Tag(soup, 'color')
            color_tag['value'] = tag['color']
            color_tag.insert(0, tag.text)
            return color_tag
        except KeyError:
            return tag.text

    html = sanitize_html(html)
    soup = BeautifulSoup(html)
    soup = swap_tags(soup, [
        ('a', convert_anchor),
        ('ul', fake_ul),
        ('font', convert_font)
    ])

    return soup.renderContents().decode('utf8')


def remove_comments(soup):
    # remove all comments from the HTML
    for comment in soup.findAll(text=lambda text: isinstance(text, Comment)):
        comment.extract()
    return soup


def swap_tags(soup, tags):
    for element, replacer in tags:
        for tag in soup.findAll(element):
            if callable(replacer):
                replacement_tag = replacer(soup, tag)
            else:
                if element == replacer:
                    continue
                replacement_tag = Tag(soup, replacer)
                replacement_tag.insert(0, tag.text)
            tag.replaceWith(replacement_tag)
    return soup


def sanitize_html(value):
    # FIXME: 'None' should never be saved as text
    if value is None:
        return ''

    # allowed tags for a Vodafone Live <CONTAINER type="data" />
    # this doubles up as a translation table. CKEditor does new-ish
    # HTML than Vodafone Live will accept. We have to translate 'em' back
    # to 'i', and 'strong' back to 'b'.
    #
    # NOTE: Order is important since <strong>'s can be inside <p>'s.
    tags = (
        ('em', 'i'),  # when creating them in the editor they're EMs
        ('strong', 'b'),
        ('i', 'i'),  # when loading them as I's the editor leaves them
        ('b', 'b'),  # we keep them here to prevent them from being removed
        ('u', 'u'),
        ('br', 'br'),
        ('p', 'p'),
        ('li', 'li'),
        ('ul', 'ul'),
        ('small', 'small'),

        # FIXME:    Because of how this is implemented this method might
        #           be called twice because our crazy template-in-a-template
        #           setup. If that's the case we'll potentially get HTML
        #           on the first round and PML on the second round.
        #           We unfortunately need to deal with both.
        ('a', 'a'),         # this allows `html_to_text` to replace them with LINKs
        ('font', 'font'),   # this allows `html_to_text` to replace them with COLORs
    )
    valid_tags = [tag for tag, replacement_tag in tags]
    soup = BeautifulSoup(value)
    soup = remove_comments(soup)

    # hide all tags that aren't in the allowed list, but keep
    # their contents
    for tag in soup.findAll(True):
        # Vodafone Live allows for no tag attributes
        # tag.attrs = []
        if tag.name not in valid_tags:
            tag.hidden = True

    # replace tags with Vlive equivelants
    soup = swap_tags(soup, tags)

    xml = soup.renderContents().decode('utf8')
    fragment, errors = tidylib.tidy_fragment(xml, {
        'char-encoding': 'utf8'
    })

    return fragment \
            .replace('&nbsp;', ' ') \
            .replace('&rsquo;', '\'') \
            .replace('&lsquo;', '\'') \
            .replace('&quot;', '"') \
            .replace('&ldquo;', '"') \
            .replace('&rdquo;', '"') \
            .replace('&ndash;', "-")\
            .replace('&copy;', "&#169;")\
            .replace('&raquo;', "&#187;")\
            .replace('&laquo;', "&#171;")\
            .replace('&bull;', "-")\
            .replace('&hellip;', "...")



from django.forms import Form
from django.template.loader import render_to_string
from pml.forms.fields import (
    PMLCheckBoxField, PMLRadioField, PMLSelectField,
    PMLTextField, PMLHiddenField
)


class PMLForm(Form):
    """
    Base PML which generates valid PML on a template {{ form }} call.

    @member header_text string: Text to display on form header.
    @member submit_text string: Text to display on submit button.
    """

    def __init__(self, *args, **kwargs):
        super(PMLForm, self).__init__(*args, **kwargs)

        # Attach errors to widgets
        for key, error in self.errors.items():
            self.fields[key].widget.error = error

    def __unicode__(self):
        """
        Renders valid PML form in its entirety.
        @returns string: safe PML.
        """
        return render_to_string("pml/form.xml", {'form': self})



from django.forms import Widget
from django.template.loader import render_to_string


class PMLChoiceBase(object):
    def render(self, name, value, attrs=None):
        context = {}
        context['attrs'] = self.attrs
        context['name'] = name
        context['value'] = value
        context['widget'] = self
        context['choices'] = self.choices
        return render_to_string(self.template, context)


class PMLCheckBoxWidget(PMLChoiceBase, Widget):
    template = "pml/widgets/choice.xml"
    
    def render(self, name, value, attrs=None):
        self.attrs.update({"type": "checkbox"})
        return super(PMLCheckBoxWidget, self).render(name, value, attrs)


class PMLRadioWidget(PMLChoiceBase, Widget):
    template = "pml/widgets/choice.xml"
    
    def render(self, name, value, attrs=None):
        self.attrs.update({"type": "radio"})
        return super(PMLRadioWidget, self).render(name, value, attrs)


class PMLSelectWidget(PMLChoiceBase, Widget):
    template = "pml/widgets/select.xml"
    

class PMLTextWidget(Widget):
    def render(self, name, value, attrs=None):
        context = {}
        context['attrs'] = self.attrs
        context['name'] = name
        context['value'] = value
        context['widget'] = self
        return render_to_string("pml/widgets/text.xml", context)


class PMLHiddenWidget(Widget):
    def render(self, name, value, attrs=None):
        self.attrs.update({"type": "hidden"})
        context = {}
        context['attrs'] = self.attrs
        context['name'] = name
        context['value'] = value
        context['widget'] = self
        return render_to_string("pml/widgets/hidden.xml", context)



from django import forms
from pml.forms import widgets

class PMLBaseField(object):
    def __init__(self, *args, **kwargs):
        super(PMLBaseField, self).__init__(*args, **kwargs)
        self.widget.label = self.label
        self.widget.help_text = self.help_text

class PMLCheckBoxField(PMLBaseField, forms.ChoiceField):
    widget = widgets.PMLCheckBoxWidget


class PMLRadioField(PMLBaseField, forms.ChoiceField):
    widget = widgets.PMLRadioWidget

    
class PMLSelectField(PMLBaseField, forms.ChoiceField):
    widget = widgets.PMLSelectWidget


class PMLTextField(PMLBaseField, forms.CharField):
    widget = widgets.PMLTextWidget


class PMLHiddenField(PMLTextField):
    widget = widgets.PMLHiddenWidget



from django.conf.urls import patterns, url, include
from django.contrib import admin

admin.autodiscover()

urlpatterns = patterns('',
    url(r'^$','pml.test_view.home', name='home'),
    url(r'^redirect/$','pml.test_view.go_back', name='go_back'),
    url(r'^admin/', include(admin.site.urls)),
)



from setuptools import setup, find_packages

setup(
    name='django-pummel',
    version='0.0.3',
    description='Django PML template tags and utils (djang-pml was taken, okay!)',
    long_description=open('README.rst', 'r').read() + open('AUTHORS.rst', 'r').read() + open('CHANGELOG.rst', 'r').read(),
    author='Praekelt Foundation',
    author_email='dev@praekeltfoundation.org',
    license='Proprietary',
    url='http://github.com/praekelt/django-pummel',
    packages=find_packages(),
    dependency_links=[
    ],
    install_requires=[
        'django',
        'lxml',
        'pytidylib',
        'BeautifulSoup>=3.0.8.1',
    ],
    tests_require=[
        'django-setuptest',
    ],
    test_suite="setuptest.setuptest.SetupTestSuite",
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    zip_safe=False,
)



from django import template
from django.template import Template
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe, SafeData
from pml import utils


class PMLTagException(Exception):
    pass


register = template.Library()


def base_builder(context):
    context['color'] = utils.resolve_color(context['color'])
    return context


@register.inclusion_tag('pml/inclusion_tags/banner.xml', takes_context=True)
def banner(context, image_url):
    t = Template(image_url)
    return {
        'image_url': t.render(context)
    }


@register.inclusion_tag('pml/inclusion_tags/divider.xml')
def divider():
    return {}


@register.inclusion_tag('pml/inclusion_tags/header.xml', takes_context=True)
def header(context, text, color=None):
    text = Template(text).render(context)
    context = {}
    context.update(locals())
    return base_builder(context)


@register.inclusion_tag('pml/inclusion_tags/list.xml')
def link_list(objects, url_property, text_property, color=None, role=None):
    context = {}
    context.update(locals())
    context = base_builder(context)

    object_list = []
    for obj in objects:
        if isinstance(obj, dict) and url_property in obj:
            url = obj[url_property]
        else:
            url = getattr(obj, url_property, '')
        if isinstance(obj, dict) and text_property in obj:
            text = obj[text_property]
        else:
            text = getattr(obj, text_property, '')
        object_list.append({
            'url': url,
            'text': text
        })
    context['object_list'] = object_list
    return context


@register.inclusion_tag('pml/inclusion_tags/list.xml')
def horizontal_links(objects, url_property, text_property, color=None):
    context = link_list(objects, url_property, text_property, color)
    context['role'] = 'MOD19'
    return context


@register.inclusion_tag('pml/inclusion_tags/thumbnail_html.xml',
    takes_context=True)
def thumbnail_html(context, image_url, html, color=None, align='left'):
    image_url = Template(image_url).render(context)
    context.update(locals())
    return base_builder(context)


@register.inclusion_tag('pml/inclusion_tags/thumbnail_html.xml',
    takes_context=True)
def thumbnail_include(context, image_url, template, color=None, align='left'):
    image_url = Template(image_url).render(context)
    context.update(locals())
    html = render_to_string(template, context)
    context['html'] = mark_safe(utils.html_to_text(html))
    return base_builder(context)


@register.inclusion_tag('pml/inclusion_tags/page_banner.xml',
    takes_context=True)
def page_banner_include(context, image_url, template, color=None, role=None):
    context.update(locals())
    html = render_to_string(template, context)
    context['html'] = mark_safe(utils.html_to_text(html))
    return base_builder(context)


@register.inclusion_tag('pml/inclusion_tags/clickable_banner.xml',
    takes_context=True)
def clickable_banner_include(context, image_url, link_url, template, color=None, role=None):
    context.update(locals())
    html = render_to_string(template, context)
    if not isinstance(html, SafeData):
        html = utils.html_to_text(html)
    context['html'] = html
    return base_builder(context)


@register.inclusion_tag('pml/inclusion_tags/link_tag.xml')
def link_tag(url, text, color=None):
    context = {}
    context.update(locals())
    return base_builder(context)


@register.inclusion_tag('pml/inclusion_tags/text_module.xml',
    takes_context=True)
def text_module(context, html=None, template=None, color=None, role=None):
    if not (html or template):
        raise PMLTagException(
            'Please provide either html or template key word argument')

    if template:
        html = Template(template).render(context)
        html = mark_safe(utils.html_to_text(html))

    context = {}
    context.update(locals())
    return base_builder(context)


@register.inclusion_tag('pml/inclusion_tags/text_module.xml',
    takes_context=True)
def text_include(context, template, color=None, role=None):
    context.update(locals())
    html = render_to_string(template, context)
    context['html'] = mark_safe(utils.html_to_text(html))
    return base_builder(context)


@register.inclusion_tag('pml/inclusion_tags/text_tag.xml', takes_context=True)
def text_tag(context, html):
    # NOTE: this makes a crazy assumption that if we receive "safe"
    #       html then it's already been html_to_text()-ed.
    #
    #       This is because text_module internally calls text_tag, this also
    #       means that html_to_text is called twice. This can cause problems
    #       because html_to_text potentially throws out tags it does not exist.
    #
    #       See https://github.com/praekelt/django-pummel/pull/4 for context
    #       as the <A> it turned into a <LINK> in the first run and removed
    #       completely on the second run.
    if not isinstance(html, SafeData):
        html = utils.html_to_text(html)
    return locals()


@register.inclusion_tag('pml/inclusion_tags/title_tag.xml', takes_context=True)
def title_tag(context, html):
    html = utils.html_to_text(html)
    return locals()


@register.inclusion_tag('pml/inclusion_tags/redirect.xml', takes_context=True)
def redirect(context, seconds, url=None):
    tenths_of_seconds = seconds * 10
    if not url:
        url = context['request'].META['HTTP_REFERER']
    return locals()
